/***********************************************************************************************************************
 *
 * blueBill Mobile - Android - open source birding
 * Copyright (C) 2009-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluebill.tidalwave.it/mobile
 * SCM: https://java.net/hg/bluebill-mobile~android-src
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebill.mobile.media.impl;

import javax.annotation.Nonnegative;
import javax.annotation.Nonnull;
import java.util.HashMap;
import java.util.Map;
import it.tidalwave.util.NotFoundException;
import it.tidalwave.util.Key;
import it.tidalwave.semantic.GenericEntity;
import it.tidalwave.mobile.util.Downloadable.Status;
import it.tidalwave.mobile.media.Media;
import it.tidalwave.bluebill.mobile.android.R;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public final class AndroidMediaNodeRenderer // FIXME: change name
  {
    public final static Map<Status, Integer> COLOR_MAP_BY_STATUS = new HashMap<Status, Integer>();

    public final static Map<String, Integer> ICON_MAP_BY_ICON_KEY = new HashMap<String, Integer>();
    
    public final static GenericEntity UNKNOWN_AUTHOR = GenericEntity.namedEntity("");
    public final static GenericEntity UNKNOWN_LOCATION = GenericEntity.namedEntity("");

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    static
      {
        COLOR_MAP_BY_STATUS.put(Status.BROKEN,         R.color.download_status_broken);
        COLOR_MAP_BY_STATUS.put(Status.DOWNLOADED,     R.color.download_status_downloaded);
        COLOR_MAP_BY_STATUS.put(Status.DOWNLOADING,    R.color.download_status_downloading);
        COLOR_MAP_BY_STATUS.put(Status.NOT_DOWNLOADED, R.color.download_status_not_downloaded);
        COLOR_MAP_BY_STATUS.put(Status.OBSOLETE,       R.color.download_status_obsolete);
        COLOR_MAP_BY_STATUS.put(Status.QUEUED,         R.color.download_status_queued);

        ICON_MAP_BY_ICON_KEY.put("download", R.drawable.download);
        ICON_MAP_BY_ICON_KEY.put("play",     android.R.drawable.ic_media_play);
        ICON_MAP_BY_ICON_KEY.put("stop",     android.R.drawable.ic_media_pause);
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    protected static <T> T get (final @Nonnull Media media, final @Nonnull Key<T> key, final @Nonnull T defaultValue)
      {
        try
          {
            return media.get(key);
          }
        catch (NotFoundException e)
          {
            return defaultValue;
          }
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    @Nonnull
    protected static String formatDuration (final @Nonnegative int duration)
      {
        return String.format("%02d:%02d", duration / 60, duration % 60);
      }
  }