/***********************************************************************************************************************
 *
 * blueBill Mobile - Android - open source birding
 * Copyright (C) 2009-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluebill.tidalwave.it/mobile
 * SCM: https://java.net/hg/bluebill-mobile~android-src
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebill.mobile.media.impl;

import it.tidalwave.netbeans.util.LazyLookup;
import it.tidalwave.semantic.Type;
import org.openide.util.Lookup;
import it.tidalwave.util.AsException;
import javax.annotation.Nonnull;
import java.util.Collection;
import java.util.Collections;
import org.openide.util.Lookup.Template;
import it.tidalwave.netbeans.capabilitiesprovider.CapabilitiesProviderSupport;
import it.tidalwave.role.ui.android.ViewRenderable;
import it.tidalwave.util.NotFoundException;
import lombok.RequiredArgsConstructor;
import static it.tidalwave.mobile.media.Media.*;

/***********************************************************************************************************************
 * 
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
@RequiredArgsConstructor
public abstract class MediaPresentationModelViewRenderableCapabilityProviderSupport extends CapabilitiesProviderSupport<MediaPresentationModel> 
  { 
    @Nonnull
    private final Type type;
    
    // We can't just directly create the capability as we want to inspect the object's Lookup, and this operation
    // can't be done in a CapabilityProvider, since the Lookup has not been instantiated yet. So we're using a lazy
    // Lookup that will postpone the operation.
    @Override @Nonnull
    public Collection<? extends Lookup> createLookups (final @Nonnull MediaPresentationModel owner) 
      {
        return Collections.singletonList(new LazyLookup() 
          {
            @Nonnull
            protected Collection<?> computeInstances (final @Nonnull Template<?> template)
              {
                final Class<?> type = template.getType();
                
                if (ViewRenderable.class.isAssignableFrom(type))
                  {
                    try
                      {
                        final String mediaType = owner.as(Media).get(TYPE); 

                        if (MediaPresentationModelViewRenderableCapabilityProviderSupport.this.type.stringValue().equals(mediaType)) 
                          {
                            return Collections.singletonList(type.cast(createViewRenderable(owner)));
                          } 
                      }
                    catch (NotFoundException e)
                      {
                        // ok fallback
                      }
                    catch (AsException e)
                      {
                        // ok fallback
                      }
                  }

                return Collections.emptyList();
              }
          });
      }    

    @Nonnull
    protected abstract ViewRenderable createViewRenderable (final @Nonnull MediaPresentationModel owner);
  }
