/***********************************************************************************************************************
 *
 * blueBill Mobile - Android - open source birding
 * Copyright (C) 2009-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluebill.tidalwave.it/mobile
 * SCM: https://java.net/hg/bluebill-mobile~android-src
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebill.mobile.media.impl;

import it.tidalwave.mobile.media.Media;
import javax.annotation.Nonnull;
import android.view.View;
import android.widget.TextView;
import it.tidalwave.util.NotFoundException;
import it.tidalwave.role.ui.PresentationModel;
import it.tidalwave.role.ui.android.ViewRenderable;
import it.tidalwave.bluebill.mobile.android.R;
import it.tidalwave.bluebill.mobile.android.taxonomy.factsheet.LicenseRenderer;
import lombok.RequiredArgsConstructor;
import static it.tidalwave.semantic.vocabulary.DublinCoreVocabulary.*;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
@RequiredArgsConstructor
public class MovingImagePresentationModelViewRenderable implements ViewRenderable
  {
    private final PresentationModel presentationModel;

    @Override @Nonnull
    public void renderTo (final @Nonnull View view, final @Nonnull Object... extra) 
      {
        final TextView tvLabel = (TextView)view.findViewById(R.id.tvLabel);
        final TextView tvLocationAndDate = (TextView)view.findViewById(R.id.tvLocationAndDate);
        final TextView tvAuthor = (TextView)view.findViewById(R.id.tvAuthor);
        final TextView tvRights = (TextView)view.findViewById(R.id.tvRights);
        final TextView tvDuration = (TextView)view.findViewById(R.id.tvDuration);

        final Media movie = presentationModel.as(Media.class);

        try
          {
            tvLabel.setText(movie.get(DC_TITLE));
          }
        catch (ClassCastException e) // FIXME: use ad hoc CardinalityException
          {
//            tvLabel.setText(movie.getMultiple(DC_TITLE).toString());
          }
        catch (NotFoundException e)
          {
            tvLabel.setText("");  
          }

        try
          {
            String movieId = movie.get(Media.ID).stringValue();
            movieId = movieId.replaceAll(".*/", "").replaceAll("#.*$", "");

            tvAuthor.setText("BBC" + " " + movieId);
          }
        catch (NotFoundException e)
          {
            tvLocationAndDate.setText("");  
          }  

        try
          {
            tvLocationAndDate.setText(movie.get(DC_ABSTRACT));
          }
        catch (NotFoundException e)
          {
            tvLocationAndDate.setText("");  
          }  

//            tvLocationAndDate.setText(location.as(Displayable).getDisplayName() + " " + df.format(date));
//            tvAuthor.setText(author.as(Displayable).getDisplayName() + ", " + source.as(Displayable).getDisplayName() + " " + movieId);
//            LicenseRenderer.renderLicense(tvRights, movie);
//            tvDuration.setText(formatDuration(media.get(Media.DURATION)));
        tvDuration.setText(AndroidMediaNodeRenderer.formatDuration(0));
      }
  }
